""" A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""
from setuptools import setup, find_packages

from codecs import open
from os import path

import galaxies.src.getters.get_global_variable as ggv


_HERE = path.abspath(path.dirname(__file__))
_README_FILE_NAME = "README.md"
_README_FILE_ENCODING = "utf8"


# Get the long description from the README file
with open(path.join(_HERE, _README_FILE_NAME),
          encoding=_README_FILE_ENCODING) as file:
    long_description = file.read()

setup(
    name=ggv.name(),

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=ggv.version(),

    description=ggv.description(),
    long_description=long_description,

    # The project's main homepage.
    url=ggv.main_homepage(),

    # Not to use with Python versions prior to 2.2.3 or 2.3
    download_url=ggv.download_url(),

    # Author details
    author=ggv.author(),
    author_email=ggv.email(),

    # Choose your license
    license=ggv.license_used(),

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=ggv.classifiers(),

    # What does your project relate to?
    keywords=ggv.keywords(),

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=ggv.dependencies(),

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={

    },

    # Set to True if we use MANIFEST.in
    include_package_data=True,

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            ggv.cli_entry_point() + ' = galaxies.main:cli_entry_point',
        ],
        'gui_scripts': [
            ggv.gui_entry_point() + ' = galaxies.main:gui_entry_point'
        ]
    }
)
